from django.db.models import Q

from aio_client.provider.api import PostProviderRequest
from aio_client.provider.api import push_request
from m3.plugins import ExtensionManager

from kinder.core.declaration.models import Declaration
from kinder.core.children.models import ChildrenDelegate

from concentrator.smev3_v321.base import constants as base_enums
from concentrator.smev3_v321.base.utils import BaseExecutor
from concentrator.smev3_v321.base.utils import SMEV3Response
from concentrator.smev3_v321.base.utils import render_type2xml
from concentrator.smev3_v321.order.helpers import (
    OrderRequestSMEV3RequestManager)
from concentrator.smev3_v321.order.request_context import OrderRequestContext
from concentrator.smev3_v321.service_types import kinder_conc

from .constants import APPLICATION_ORDER_INFO_REQUEST
from .constants import APPLICATION_ORDER_INFO_REQUEST_DELEGATE_ERROR_COMMENT
from .constants import APPLICATION_ORDER_INFO_REQUEST_SUCCESS_CODE
from .constants import APPLICATION_ORDER_INFO_REQUEST_SUCCESS_COMMENT
from .constants import APPLICATION_ORDER_INFO_REQUEST_ERROR_CODE
from .constants import APPLICATION_ORDER_INFO_REQUEST_ERROR_COMMENT
from .utils import check_and_update_decls_delegates


class ApplicationOrderInfoRequestExecutor(BaseExecutor):
    """Исполнитель сервиса ApplicationOrderInfoRequest."""

    name_service = APPLICATION_ORDER_INFO_REQUEST
    type_service = kinder_conc.ApplicationOrderInfoRequestType

    @staticmethod
    def create_response(order_id, tech_code, comment, message):
        response_body = render_type2xml(
            kinder_conc.FormDataResponseType(
                changeOrderInfo=kinder_conc.changeOrderInfoType(
                    orderId=kinder_conc.orderIdType(order_id),
                    statusCode=kinder_conc.statusCodeType(
                        techCode=tech_code),
                    comment=comment)),
            name_type='FormDataResponse')

        return push_request(PostProviderRequest(
            origin_message_id=message['origin_message_id'],
            body=response_body,
            message_type=message['message_type'],
            replay_to=message['replay_to'],
            is_test_message=message['is_test_message'])), response_body

    @classmethod
    def process(cls, message, request_body, **kwargs):

        order_id = request_body.ApplicationOrderInfoRequest.orderId
        child_info = request_body.ApplicationOrderInfoRequest.ChildInfo
        d_dul = request_body.ApplicationOrderInfoRequest.PersonIdentityDocInfo

        # Фильтр для поиска заявления по ФИ ребенка, Дате рождения
        # и ДУЛ (серия, номер).
        base_filter = (
                Q(children__surname__iexact=child_info.ChildSurname)
                & Q(children__firstname__iexact=child_info.ChildName)
                & Q(children__date_of_birth=child_info.ChildBirthDate)
                & Q(children__dul_series=(
                    child_info.ChildBirthDocRF.ChildBirthDocSeries))
                & Q(children__dul_number=(
                    child_info.ChildBirthDocRF.ChildBirthDocNumber)))

        declarations_query = Declaration.objects.filter(base_filter)

        if not declarations_query.exists():
            tech_code = APPLICATION_ORDER_INFO_REQUEST_ERROR_CODE
            comment = APPLICATION_ORDER_INFO_REQUEST_ERROR_COMMENT
        else:
            # Поверка наличия нужных данных у представителя и попытка
            # обновления их данными из запроса
            decls_query_with_delegates = check_and_update_decls_delegates(
                declarations_query, request_body.ApplicationOrderInfoRequest)

            if not decls_query_with_delegates.exists():
                tech_code = APPLICATION_ORDER_INFO_REQUEST_ERROR_CODE
                comment = APPLICATION_ORDER_INFO_REQUEST_DELEGATE_ERROR_COMMENT
            else:
                tech_code = APPLICATION_ORDER_INFO_REQUEST_SUCCESS_CODE
                comment = APPLICATION_ORDER_INFO_REQUEST_SUCCESS_COMMENT

                for declaration in decls_query_with_delegates.iterator():
                    chdel_set = declaration.children.childrendelegate_set

                    zags_act_number = (
                        child_info.ChildBirthDocRF.ChildBirthDocActNumber)

                    ExtensionManager().execute(
                        'kinder.plugins.tatarstan_specifics.extensions'
                        '.update_child_zags_act_number',
                        declaration, zags_act_number)

                    try:
                        delegate_id = chdel_set.get(
                            delegate__dul_series=d_dul.IdentityDocSeries,
                            delegate__dul_number=d_dul.IdentityDocNumber
                        ).delegate_id
                    except ChildrenDelegate.DoesNotExist:
                        delegate_id = None

                    OrderRequestSMEV3RequestManager(
                        OrderRequestContext(
                            **{
                                'declaration_id': declaration.id,
                                'order_id': order_id,
                                'delegate_id': delegate_id,
                            })).apply_async()

        response, response_body = cls.create_response(
            order_id, tech_code, comment, message)

        return SMEV3Response(
            response,
            {
                'method_name': cls.name_service,
                'response': response_body,
                'result': base_enums.SUCCESS_MESSAGE
            })
